@@ -1,5 +1,5 @@
 from typing import Tuple, Dict, Any
-from web3 import Web3
+from web3 import HTTPProvider, Web3
 from eth_utils.abi import function_abi_to_4byte_selector, collapse_if_tuple
 from actions_lib.actions.type import Action, ActionData
 from actions_lib.utils.contact_tool import get_contact_address
@@ -55,6 +55,11 @@ ERC20_APPROVE_ABI = """[{"constant":false,"inputs":[{"name":"_spender","type":"a
 
 ERC20_TRANSFER_ABI = """[{"constant":false,"inputs":[{"name":"recipient","type":"address"},{"name":"amount","type":"uint256"}],"name":"transfer","outputs":[{"name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"}]"""
 
+def get_web3(providers, chain):
+    if providers[chain]['w3'] is None:
+        providers[chain]['w3'] = Web3(HTTPProvider(providers[chain]['rpc']))
+    return providers[chain]['w3']
+
 def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', chain: str = "base", token: str = "usdc", **kwargs: Any) -> Dict[str, Any]:
     print(f"receiver: {receiver}, amount: {amount}, run_mode: {run_mode}, chain: {chain}, token: {token}, kwargs: {kwargs}, step: {step}")
     
@@ -62,9 +67,6 @@ def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', ch
     executor = kwargs.get("executor")
     redis_client = kwargs.get("redis_client")
     spender_info = kwargs.get('spender')
-    # control_contract = kwargs.get('contract')
-    # private_key = kwargs.get('private_key')
-    # spender_address = kwargs.get('spender_address')
     providers = kwargs.get("providers")
     
     if not providers or chain_raw not in providers:
@@ -74,10 +76,9 @@ def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', ch
     control_contract = spender_info[chain_raw]['control_contract']
     sender_address = spender_info[chain_raw]['sender_address']
     private_key = spender_info[chain_raw]['sender_private_key']
-    
     blockchain_explore = providers[chain_raw]['scan']
-    w3 = providers[chain_raw]['w3']
-    eth_w3 = providers['eth']['w3']
+    w3 = get_web3(providers, chain_raw)
+    eth_w3 = get_web3(providers, 'eth')
     token_raw = token.lower()
     token_address = w3.to_checksum_address(TOKEN_MAPPING[chain_raw][token_raw])
     
@@ -125,8 +126,12 @@ def handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address,
 def handle_manual_transfer(w3, executor, receiver, amount, token_address, redis_client, chain_raw, token_raw):
     active_chain_key = f"user:{executor}:active_chain"
     active_chain = redis_client.get(active_chain_key) or "base"
+    print(f"active_chain_key: {active_chain_key}, active_chain: {active_chain}", flush=True)
     
-    if active_chain.lower() != chain_raw:
+    active_chain_id = CHAIN_ID_MAPPING[active_chain.lower()]
+    param_chain_id = CHAIN_ID_MAPPING[chain_raw]
+    
+    if param_chain_id != active_chain_id:
         return handle_incorrect_network(w3, executor, receiver, amount, chain_raw, token_raw, redis_client)
     
     code, receiver_address = get_contact_address(redis_client, executor, receiver, w3)
@@ -196,16 +201,19 @@ def handle_incorrect_network(w3, executor, receiver, amount, chain_raw, token_ra
 
     return {'result': {'code': 200, 'content': 'Incorrect network, switching to the correct network for you.'}, 'action': action.__dict__, 'next_action': next_action.__dict__}
 
-def send_transaction(w3, control_contract, sender_address, token, executor, to, amount_to_transfer, private_key):
+def send_transaction(w3: Web3, control_contract, sender_address, token, executor, to, amount_to_transfer, private_key):
     nonce = w3.eth.get_transaction_count(sender_address)
     current_gas_price = w3.eth.gas_price
+    increase = current_gas_price * 25  // 100
+    new_gas_price = current_gas_price + increase
     tx = control_contract.functions.transferFromERC20(
         token, executor, to, amount_to_transfer).build_transaction({
             'from': sender_address,
             'nonce': nonce,
-            'gas': 200000,
-            'gasPrice': current_gas_price,
+            'gasPrice': new_gas_price,
         })
+    gas_estimate = w3.eth.estimate_gas(tx)
+    tx['gas'] = gas_estimate
     signed_tx = w3.eth.account.sign_transaction(tx, private_key)
     return w3.eth.send_raw_transaction(signed_tx.rawTransaction)
 
